@@ -4,7 +4,6 @@ Structure, Field, StructureReference, ClassReference, TypedField
 """
 import enum
 import inspect
-import json
 from builtins import issubclass
 from copy import deepcopy
 from collections import OrderedDict, deque, defaultdict
@@ -15,7 +14,7 @@ import hashlib
 
 from typing import get_type_hints, Iterable
 
-from .commons import wrap_val, _is_sunder, _is_dunder
+from .commons import raise_errs_if_needed, wrap_val, _is_sunder, _is_dunder
 from .utility import type_is_generic
 
 REQUIRED_FIELDS = "_required"
@@ -63,7 +62,7 @@ class ImmutableMixin:
     def _raise_if_immutable(self):
         if self._is_immutable():
             name = getattr(self, "_name", None)
-            raise ValueError("{}: Field is immutable".format(name))
+            raise ValueError(f"{name}: Field is immutable")
 
 
 def make_signature(
@@ -155,30 +154,24 @@ def _check_for_final_violations(classes):
     def is_sub_class(c, base):
         return issubclass(c, base) and c != base
 
-    current_class, *inherited_class = classes
-    for i, c in enumerate(inherited_class):
+    _, *inherited_class = classes
+    for c in inherited_class:
         if "FinalStructure" in globals() and isinstance(c, StructMeta):
             if "FinalStructure" in globals() and is_sub_class(c, FinalStructure):
                 raise TypeError(
-                    "Tried to extend {}, which is a FinalStructure. This is forbidden".format(
-                        c.__name__
-                    )
+                    f"Tried to extend {c.__name__}, which is a FinalStructure. This is forbidden"
                 )
             if "ImmutableStructure" in globals() and is_sub_class(
                 c, ImmutableStructure
             ):
                 raise TypeError(
-                    "Tried to extend {}, which is an ImmutableStructure. This is forbidden".format(
-                        c.__name__
-                    )
+                    f"Tried to extend {c.__name__}, which is an ImmutableStructure. This is forbidden"
                 )
 
         if "_FieldMeta" in globals() and isinstance(c, _FieldMeta):
             if "ImmutableField" in globals() and is_sub_class(c, ImmutableField):
                 raise TypeError(
-                    "Tried to extend {}, which is an ImmutableField. This is forbidden".format(
-                        c.__name__
-                    )
+                    f"Tried to extend {c.__name__}, which is an ImmutableField. This is forbidden"
                 )
 
 
@@ -216,13 +209,13 @@ class _FieldMeta(type):
 
                 if not isinstance(val, type):
                     raise TypeError(
-                        "Unsupported field type in definition: {}".format(wrap_val(val))
+                        f"Unsupported field type in definition: {wrap_val(val)}"
                     )
                 the_class = val.__name__
                 if the_class in _FieldMeta._registry:
                     return _FieldMeta._registry[the_class]
                 short_hash = hashlib.sha256(the_class.encode("utf-8")).hexdigest()[:8]
-                new_name = "Field_{}_{}".format(the_class, short_hash)
+                new_name = f"Field_{the_class}_{short_hash}"
                 class_as_field = create_typed_field(new_name, val)
                 class_as_field.__getstate__ = get_state
                 _FieldMeta._registry[the_class] = class_as_field
@@ -244,9 +237,7 @@ class UniqueMixin:
             if hash_of_instance in getattr(myclass, "_ALL_INSTANCES", set()):
                 classname = self.__class__.__name__
                 raise ValueError(
-                    "Instance copy in {}, which is defined as unique. Instance is {}".format(
-                        classname, self
-                    )
+                    f"Instance copy in {classname}, which is defined as unique. Instance is {self}"
                 )
             getattr(myclass, "_ALL_INSTANCES", set()).add(hash_of_instance)
 
@@ -272,10 +263,8 @@ class UniqueMixin:
                 != instance
             ):
                 raise ValueError(
-                    "Instance copy of field {} in {}, which is defined as unique. "
-                    "Instance is {}".format(
-                        field_name, structure_class_name, wrap_val(value)
-                    )
+                    f"Instance copy of field {field_name} in {structure_class_name}, which is defined as unique. "
+                    f"Instance is {wrap_val(value)}"
                 )
             if hash_of_field_val not in instance_by_value_for_current_struct:
                 instance_by_value_for_current_struct[hash_of_field_val] = instance
@@ -335,9 +324,7 @@ class Field(UniqueMixin, metaclass=_FieldMeta):
             self.__set__(Structure(), default)
         except Exception as e:
             raise e.__class__(
-                "Invalid default value: {}; Reason: {}".format(
-                    wrap_val(default), str(e)
-                )
+                f"Invalid default value: {wrap_val(default)}; Reason: {str(e)}"
             ) from e
 
     def __get__(self, instance, owner):
@@ -370,7 +357,7 @@ class Field(UniqueMixin, metaclass=_FieldMeta):
 
     def __set__(self, instance, value):
         if getattr(self, IS_IMMUTABLE, False) and self._name in instance.__dict__:
-            raise ValueError("{}: Field is immutable".format(self._name))
+            raise ValueError(f"{self._name}: Field is immutable")
         if getattr(self, IS_IMMUTABLE, False) and not getattr(
             self, "_custom_deep_copy_implementation", False
         ):
@@ -378,9 +365,7 @@ class Field(UniqueMixin, metaclass=_FieldMeta):
                 instance.__dict__[self._name] = deepcopy(value)
             except TypeError:
                 raise TypeError(
-                    "{} cannot be immutable, as its type does not support pickle.".format(
-                        self._name
-                    )
+                    f"{self._name} cannot be immutable, as its type does not support pickle."
                 )
         else:
             self.__manage_uniqueness_for_field__(instance, value)
@@ -402,18 +387,18 @@ class Field(UniqueMixin, metaclass=_FieldMeta):
             :return: a string representation
             """
             if hasattr(the_val, "__iter__"):
-                return "[{}]".format(", ".join([str(v) for v in the_val]))
+                return f"[{', '.join([str(v) for v in the_val])}]"
             return str(the_val)
 
         name = self.__class__.__name__
         props = []
         for k, val in sorted(self.__dict__.items()):
             if val is not None and not k.startswith("_"):
-                strv = "'{}'".format(val) if isinstance(val, str) else as_str(val)
-                props.append("{} = {}".format(k, strv))
+                strv = f"'{val}'" if isinstance(val, str) else as_str(val)
+                props.append(f"{k} = {strv}")
 
-        propst = ". Properties: {}".format(", ".join(props)) if props else ""
-        return "<{}{}>".format(name, propst)
+        propst = f". Properties: {', '.join(props)}" if props else ""
+        return f"<{name}{propst}>"
 
     def _set_immutable(self, immutable: bool):
         self._immutable = immutable
@@ -522,7 +507,7 @@ class StructMeta(type):
         fields = [key for key, val in cls_dict.items() if isinstance(val, Field)]
         for field_name in fields:
             if field_name.startswith("_") or field_name == "kwargs":
-                raise ValueError("{}: invalid field name".format(field_name))
+                raise ValueError(f"{field_name}: invalid field name")
             setattr(cls_dict[field_name], "_name", field_name)
 
         _apply_default_and_update_required_not_to_include_fields_with_defaults(
@@ -561,9 +546,10 @@ class StructMeta(type):
         props = []
         for k, val in sorted(cls.__dict__.items()):
             if val is not None and not k.startswith("_"):
-                strv = "'{}'".format(val) if isinstance(val, str) else str(val)
-                props.append("{} = {}".format(k, strv))
-        return "<Structure: {}. Properties: {}>".format(name, ", ".join(props))
+                strv = f"'{val}'" if isinstance(val, str) else str(val)
+                props.append(f"{k} = {strv}")
+        props_list = ", ".join(props)
+        return f"<Structure: {name}. Properties: {props_list}>"
 
 
 def convert_basic_types(v):
@@ -608,12 +594,16 @@ def get_typing_lib_info(v):
         return v
     if inspect.isclass(v) and issubclass(v, Field):
         return v()
+
+    if isinstance(v, StructMeta) and not isinstance(v, Field):
+        return ClassReference(v)
+
     if not type_is_generic(v):
         return convert_basic_types(v)
     origin = getattr(v, "__origin__", None)
     mapped_type = convert_basic_types(origin)
     if mapped_type is None:
-        raise TypeError("{} type is not supported".format(v))
+        raise TypeError(f"{v} type is not supported")
     args_raw = getattr(v, "__args__", None)
     if not args_raw:
         return mapped_type()
@@ -627,9 +617,9 @@ def get_typing_lib_info(v):
                     if isinstance(args_raw[i], type):
                         mapped_args[i] = Field[args_raw[i]]
                     else:
-                        raise TypeError("invalid type {}".format(v))
+                        raise TypeError(f"invalid type {v}")
         else:
-            raise TypeError("invalid type {}".format(v))
+            raise TypeError(f"invalid type {v}")
     if mapped_args:
         if mapped_type == AnyOf:
             return mapped_type(fields=mapped_args)
@@ -645,7 +635,7 @@ def add_annotations_to_class_dict(cls_dict, previous_frame):
     optional_fields = set(cls_dict.get(OPTIONAL_FIELDS, []))
     if isinstance(annotations, dict):
         for k, v in annotations.items():
-            if isinstance(v, str):
+            if isinstance(v, str) and len(v) < 50:
                 # The evil eval is to accommodate "from __future__ import annotations".
                 v = eval(
                     v,
@@ -678,7 +668,7 @@ def add_annotations_to_class_dict(cls_dict, previous_frame):
                             else:
                                 the_type = the_type(default=default_value)
                         except Exception as e:
-                            raise e.__class__("{}: {}".format(k, str(e))) from e
+                            raise e.__class__(f"{k}: {str(e)}") from e
                         defaults[k] = cls_dict[k]
                     cls_dict[k] = the_type
     if optional_fields:
@@ -774,8 +764,8 @@ class Structure(UniqueMixin, metaclass=StructMeta):
         field_by_name = self.get_all_fields_by_name()
         defaults_fields = [
             key
-            for key in field_by_name
-            if getattr(field_by_name[key], "_default", None) is not None
+            for key, value in field_by_name.items()
+            if getattr(value, "_default", None) is not None
             and key not in bound.arguments
         ]
         for field_name in defaults_fields:
@@ -793,9 +783,7 @@ class Structure(UniqueMixin, metaclass=StructMeta):
                     setattr(self, name, val)
                 except (TypeError, ValueError) as ex:
                     errors.append(ex)
-            if errors:
-                messages = json.dumps([str(e) for e in errors])
-                raise errors[0].__class__(messages) from errors[0]
+            raise_errs_if_needed(errors)
 
         self.__validate__()
         self._instantiated = True
@@ -845,20 +833,18 @@ class Structure(UniqueMixin, metaclass=StructMeta):
             return ",".join(as_strings)
 
         def dict_to_str(values):
-            as_strings = [
-                "{} = {}".format(to_str(k), to_str(v)) for (k, v) in values.items()
-            ]
+            as_strings = [f"{to_str(k)} = {to_str(v)}" for (k, v) in values.items()]
             return ",".join(as_strings)
 
         def to_str(the_val):
             if isinstance(the_val, list):
-                return "[{}]".format(list_to_str(the_val))
+                return f"[{list_to_str(the_val)}]"
             if isinstance(the_val, tuple):
-                return "({})".format(list_to_str(the_val))
+                return f"({list_to_str(the_val)})"
             if isinstance(the_val, set):
-                return "{{{}}}".format(list_to_str(the_val))
+                return f"{{{list_to_str(the_val)}}}"
             if isinstance(the_val, dict):
-                return "{{{}}}".format(dict_to_str(the_val))
+                return f"{{{dict_to_str(the_val)}}}"
             return str(the_val)
 
         name = self.__class__.__name__
@@ -870,9 +856,10 @@ class Structure(UniqueMixin, metaclass=StructMeta):
         internal_props = ["_instantiated"]
         for k, val in sorted(self.__dict__.items()):
             if k not in internal_props:
-                strv = "'{}'".format(val) if isinstance(val, str) else to_str(val)
-                props.append("{} = {}".format(k, strv))
-        return "<Instance of {}. Properties: {}>".format(name, ", ".join(props))
+                strv = f"'{val}'" if isinstance(val, str) else to_str(val)
+                props.append(f"{k} = {strv}")
+        props_list = ", ".join(props)
+        return f"<Instance of {name}. Properties: {props_list}>"
 
     def __repr__(self):
         return self.__str__()
@@ -899,7 +886,7 @@ class Structure(UniqueMixin, metaclass=StructMeta):
         if isinstance(getattr(self, REQUIRED_FIELDS), list) and key in getattr(
             self, REQUIRED_FIELDS
         ):
-            raise ValueError("{} is mandatory".format(key))
+            raise ValueError(f"{key} is mandatory")
         del self.__dict__[key]
 
     def __validate__(self):
@@ -928,7 +915,7 @@ class Structure(UniqueMixin, metaclass=StructMeta):
     def __bool__(self):
         internal_props = ["_instantiated"]
         return any(
-            [v is not None for k, v in self.__dict__.items() if k not in internal_props]
+            v is not None for k, v in self.__dict__.items() if k not in internal_props
         )
 
     @classmethod
@@ -974,9 +961,7 @@ class Structure(UniqueMixin, metaclass=StructMeta):
             field_names = list(field_by_name.keys())
             return item in getattr(self, field_names[0], {})
 
-        raise TypeError(
-            "{} does not support this operator".format(self.__class__.__name__)
-        )
+        raise TypeError(f"{self.__class__.__name__} does not support this operator")
 
     def __iter__(self):
         field_by_name = _get_all_fields_by_name(self.__class__)
@@ -985,19 +970,13 @@ class Structure(UniqueMixin, metaclass=StructMeta):
 
         if self._is_wrapper() and hasattr(val, "__iter__"):
             return iter(val)
-        raise TypeError(
-            "{} is not a wrapper of an iterable".format(self.__class__.__name__)
-        )
+        raise TypeError(f"{self.__class__.__name__} is not a wrapper of an iterable")
 
     def shallow_clone_with_overrides(self, **kw):
         fields_names = self.get_all_fields_by_name().keys()
-        field_value_by_name = dict(
-            [
-                (f, getattr(self, f))
-                for f in fields_names
-                if getattr(self, f) is not None
-            ]
-        )
+        field_value_by_name = {
+            f: getattr(self, f) for f in fields_names if getattr(self, f) is not None
+        }
         kw_args = {**field_value_by_name, **kw}
         return self.__class__(**kw_args)
 
@@ -1023,13 +1002,11 @@ class Structure(UniqueMixin, metaclass=StructMeta):
             )
 
             fields_names = cls.get_all_fields_by_name().keys()
-            field_value_by_name = dict(
-                [
-                    (f, getattr(that, f))
-                    for f in fields_names
-                    if getattr(that, f, None) is not None
-                ]
-            )
+            field_value_by_name = {
+                f: getattr(that, f)
+                for f in fields_names
+                if getattr(that, f, None) is not None
+            }
             return cls(**field_value_by_name)
 
         raise TypeError(f"cls must be subclass of {self.__class__.__name__}")
@@ -1188,12 +1165,10 @@ class TypedField(Field):
 
     def _validate(self, value):
         def err_prefix():
-            return "{}: ".format(self._name) if self._name else ""
+            return f"{self._name}: " if self._name else ""
 
         if not isinstance(value, self._ty):
-            raise TypeError(
-                "{}Expected {}; Got {}".format(err_prefix(), self._ty, wrap_val(value))
-            )
+            raise TypeError(f"{err_prefix()}Expected {self._ty}; Got {wrap_val(value)}")
 
     def __set__(self, instance, value):
         if not getattr(instance, "_skip_validation", False):
@@ -1278,7 +1253,7 @@ class ClassReference(TypedField):
         super().__init__(cls)
 
     def __str__(self):
-        return "<ClassReference: {}>".format(self._ty.__name__)
+        return f"<ClassReference: {self._ty.__name__}>"
 
 
 class ImmutableField(Field):
